/**********************************************************************************
* C Library for the display modules
* XV-GDM128X64 and XV-GDM128X64-RGB from XV-Display.de
*
* Section: Display.c
*
* Revision 1
* 2013 Feb. 07
*
* AVR and AVR Xmega microcontrollers,
* SoftwareSPI and HardwareSPI supported.
*
*
* Recommended development environment:
*
* Atmel Studio 6
* ---> http://www.atmel.com/Microsite/atmel_studio6/default.aspx
* Developmentboard Nano from dev-tools.de.
* ---> http://www.dev-tools.de/index.php/development-boards/developmentboard-nano
*
* We invests time and resources providing this open source code,
* please support www.avr-praxis.de and open-source hardware
* by purchasing products from www.mikrocontroller-praxis.de!
*
* Written by Dirk Rodenhausen (dirk at avr-praxis.de)
*
* Our webpages:
* http://www.avr-praxis.de
* German discussion forum related to AVR microcontroller family.
* http://www.mikrocontroller-praxis.de
* Onlineshop where you can purchase the XV-Display display modules and
* the Dev-Tools Nano development board.
* http://www.dev-tools.de
* Further resources of products available from www.mikrocontroller-praxis.de.
*
*
* BSD license, all text above must be included in any redistribution.
* Please read the License.txt file.
* AVR is a registered trademark of Atmel corporation.
**********************************************************************************/


#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <stdlib.h>

#include "Display.h"
#include "Font.h"
#include "Image.h"


        #if (ARCH == ARCH_AVR8)
			#include "Display_Interface_AVR8.h"
        #elif (ARCH == ARCH_XMEGA)
			#include "Display_Interface_XMEGA.h"
        #else
            #error The peripheral driver is not currently available for your selected architecture.
        #endif


// prototypes of private library functions
void _Display_InitDisplayModule(void);
void _Display_SendCommand(uint8_t cmd);
void _Display_SendData(uint8_t data);



void Display_Init(void)
{

#if DISPLAY_BACKLIGHT_CONTROL == true
   _Display_InitBacklight();
#endif

	_Display_InitInterface();

	_Display_InitDisplayModule();
		
	SetPenMode(pmSET);						// pmSET or pmCLEAR or pmEXOR
	SetFont(FONT_1);
	SetDisplayMode(dmLandscape);			// dmLandscape or tmPortrait
	SetTextMode(tmInit);					// normal and solid brush
	
}




#if DISPLAY_BACKLIGHT_CONTROL == true

void Display_SetBacklight_By_Color(uint8_t color)
{
	
	switch (color)
	{
		case blRED:
		DISPLAY_BACKLIGHT_RED_ON;
		DISPLAY_BACKLIGHT_GREEN_OFF;		
		DISPLAY_BACKLIGHT_BLUE_OFF;	
		break;
		
		case blGREEN:
		DISPLAY_BACKLIGHT_RED_OFF;
		DISPLAY_BACKLIGHT_GREEN_ON;
		DISPLAY_BACKLIGHT_BLUE_OFF;
		break;
	  
		case blBLUE:
		DISPLAY_BACKLIGHT_RED_OFF;
		DISPLAY_BACKLIGHT_GREEN_OFF;
		DISPLAY_BACKLIGHT_BLUE_ON;
		break;	  
	  
		case blYELLOW:
		DISPLAY_BACKLIGHT_RED_ON;
		DISPLAY_BACKLIGHT_GREEN_ON;
		DISPLAY_BACKLIGHT_BLUE_OFF;				
		break;	  
	  
		case blPURPLE:
		DISPLAY_BACKLIGHT_RED_ON;
		DISPLAY_BACKLIGHT_GREEN_OFF;
		DISPLAY_BACKLIGHT_BLUE_ON;
		break;	  
	  
		case blCYAN:
		DISPLAY_BACKLIGHT_RED_OFF;
		DISPLAY_BACKLIGHT_GREEN_ON;
		DISPLAY_BACKLIGHT_BLUE_ON;				
		break;	  
	  
		case blWHITE:
		DISPLAY_BACKLIGHT_RED_ON;
		DISPLAY_BACKLIGHT_GREEN_ON;
		DISPLAY_BACKLIGHT_BLUE_ON;		
		break;	  
	  
	    default:
		DISPLAY_BACKLIGHT_RED_OFF;
		DISPLAY_BACKLIGHT_GREEN_OFF;
		DISPLAY_BACKLIGHT_BLUE_OFF;
		
	}  	  	
	
}

#endif

void SetTextMode(uint8_t mode)
{
	
	switch (mode)
	{
		
		case tmNormal:
			TextMode |= 1<<tmNormal_bp;
			break;
		
		case tmInverse:
			TextMode &= ~(1<<tmNormal_bp);
			break;
		
		case tmBrushSolid:
			TextMode |= 1<<tmBrushSolid_bp;
			break;
		
		case tmBrushClear:
			TextMode &= ~(1<<tmBrushSolid_bp);
			break;
		
		default:
			TextMode |= (1<<tmNormal_bp) | (1<<tmBrushSolid_bp);
	}
}

void Display_Clear(void)
{
	
	for (uint8_t x=0; x<128; x++)
	  for (uint8_t y=0; y<8; y++)
	    display_memory[x][y] = 0;

}

void Display_Fill(void)
{
	
	for (uint8_t x=0; x<128; x++)
	  for (uint8_t y=0; y<8; y++)
	    display_memory[x][y] = 0xFF;

}




void Display_DrawPixel(int16_t x, int16_t y)
{
	
	
	if (dmLandscape == DisplayMode)
	{
		// Landscape
		if ( (x<0) || (y<0) || (x>=DISPLAY_WIDTH) || (y>=DISPLAY_HEIGHT) ) return;
	
		if (pmSET == PenMode)			display_memory[x][y>>3] |= 0x80>>(y & 0x07);
		else if (pmCLEAR == PenMode)	display_memory[x][y>>3] &= ~(0x80>>(y & 0x07));
		else if (pmEXOR == PenMode)		display_memory[x][y>>3] ^= 0x80>>(y & 0x07);		
			
	} else {
		// Portrait		
		if ( (x<0) || (y<0) || (y>=DISPLAY_WIDTH) || (x>=DISPLAY_HEIGHT) ) return;
		
		if (pmSET == PenMode)			display_memory[y][7-(x>>3)] |= 0x01<<(x & 0x07);
		else if (pmCLEAR == PenMode)	display_memory[y][7-(x>>3)] &= ~(0x01<<(x & 0x07));
		else if (pmEXOR == PenMode)		display_memory[y][7-(x>>3)] ^= 0x01<<(x & 0x07);		
				
	}			
}

void _Display_DrawPixelChar(int16_t x, int16_t y, uint8_t mode)
{
	
	
	if (dmLandscape == DisplayMode)
	{
		// Landscape
		if ( (x<0) || (y<0) || (x>=DISPLAY_WIDTH) || (y>=DISPLAY_HEIGHT) ) return;
	    				
		if ((1<<tmNormal_bp) & TextMode)
		{
		  // Normal
		  if ((1<<tmBrushSolid_bp) & TextMode)
		  {
				if (0 == mode)
					display_memory[x][y>>3] |= 0x80>>(y & 0x07);
				else
					display_memory[x][y>>3] &= ~(0x80>>(y & 0x07));
		  }	else {
				if (0 == mode)
					display_memory[x][y>>3] |= 0x80>>(y & 0x07);  			 			  
		  }	 
		} else {
		  // Inverse
		  if ((1<<tmBrushSolid_bp) & TextMode)
		  {
				if (0 == mode)
					display_memory[x][y>>3] &= ~(0x80>>(y & 0x07));
				else
					display_memory[x][y>>3] |= 0x80>>(y & 0x07);
		  }	else {
				if (0 == mode)
					display_memory[x][y>>3] &= ~(0x80>>(y & 0x07));
		  }									
		}		
			
			
	} else {
		// Portrait		
		if ( (x<0) || (y<0) || (y>=DISPLAY_WIDTH) || (x>=DISPLAY_HEIGHT) ) return;
		
		if ((1<<tmNormal_bp) & TextMode)
		{
		  // Normal
		  if ((1<<tmBrushSolid_bp) & TextMode)
		  {
				if (0 == mode)
					display_memory[y][7-(x>>3)] |= 0x01<<(x & 0x07);
				else
					display_memory[y][7-(x>>3)] &= ~(0x01<<(x & 0x07));
		  }	else {
				if (0 == mode)
					display_memory[y][7-(x>>3)] |= 0x01<<(x & 0x07);		 			  
		  }	 
		} else {
		  // Inverse
		  if ((1<<tmBrushSolid_bp) & TextMode)
		  {
				if (0 == mode)
					display_memory[y][7-(x>>3)] &= ~(0x01<<(x & 0x07));
				else
					display_memory[y][7-(x>>3)] |= 0x01<<(x & 0x07);
		  }	else {
				if (0 == mode)
					display_memory[y][7-(x>>3)] &= ~(0x01<<(x & 0x07));
		  }									
		}								
				
	}			
}





// draw a rectangle
void Display_DrawRect(int16_t x, int16_t y, int16_t w, int16_t h)
{
	Display_DrawLine(x, y, x+w-1, y);
	Display_DrawLine(x, y+h-1, x+w-1, y+h-1);
	Display_DrawLine(x, y, x, y+h-1);
	Display_DrawLine(x+w-1, y, x+w-1, y+h-1);
}

void Display_DrawRect_Fill(int16_t x, int16_t y, int16_t w, int16_t h)
{

	for (uint16_t i=y; i<(y+h); i++) Display_DrawLine(x, i, x+w-1, i);
}

// Bresenham algorithm (http://de.wikipedia.org/wiki/Bresenham-Algorithmus)

void Display_DrawLine(int16_t x0, int16_t y0, int16_t x1, int16_t y1)
{
  int16_t i, t;
  
  i = abs(y1 - y0) > abs(x1 - x0);
  
  if (i)
  {
	  t = y0;
	  y0 = x0;
	  x0 = t;
	
	  t = y1;
	  y1 = x1;
	  x1 = t;	
  }

  if (x0 > x1)
  {
	 t = x1;
	 x1 = x0;
	 x0 = x1;	  
	
	 t = y1;
	 y1 = y0;
	 y0 = y1;
  }

  int16_t dx, dy;
  dx = x1 - x0;
  dy = abs(y1 - y0);

  int16_t err = dx / 2;
  int16_t ystep;

  if (y0 < y1) {
    ystep = 1;
  } else {
    ystep = -1;
  }

  for (; x0<=x1; x0++) {
    if (i) {
      Display_DrawPixel(y0, x0);
    } else {
      Display_DrawPixel(x0, y0);
    }
    err -= dy;
    if (err < 0) {
      y0 += ystep;
      err += dx;
    }
  }
}


void Display_DrawCircle(int16_t x0, int16_t y0, int16_t radius)
{


	int16_t f = 1 - radius;
	int16_t ddF_x = 1;
	int16_t ddF_y = -2 * radius;
	int16_t x = 0;
	int16_t y = radius;

	Display_DrawPixel(x0, y0+radius);
	Display_DrawPixel(x0, y0-radius);
	Display_DrawPixel(x0+radius, y0);
	Display_DrawPixel(x0-radius, y0);

	while (x<y) {
		if (f >= 0) {
			y--;
			ddF_y += 2;
			f += ddF_y;
		}
		x++;
		ddF_x += 2;
		f += ddF_x;
		
		Display_DrawPixel(x0 + x, y0 + y);
		Display_DrawPixel(x0 - x, y0 + y);
		Display_DrawPixel(x0 + x, y0 - y);
		Display_DrawPixel(x0 - x, y0 - y);
		Display_DrawPixel(x0 + y, y0 + x);
		Display_DrawPixel(x0 - y, y0 + x);
		Display_DrawPixel(x0 + y, y0 - x);
		Display_DrawPixel(x0 - y, y0 - x);
		
	}
}


// Image data:
// Byte 0: width in bytes (0..255)
// Byte 1: height in pixels (0..255)
// Byte 2..n: Pixel data (a byte holds pixels for a row, MSB left)

void _Display_ImagePixel(int16_t x, int16_t y, uint8_t mode);

void Display_DrawImage(int16_t x1, int16_t y1, const uint8_t *image)
{
	
	uint16_t index, w, h;
	uint8_t  width, height, i, data;
	index = 0;
	
	width = pgm_read_byte(&image[index++]);  // in bytes
	height = pgm_read_byte(&image[index++]); // in pixels (rows)
	
	for (h=0; h<height; h++)
	{
			
	  for (w=0; w<(width<<3); w+=8)
	  {
		  data = pgm_read_byte(&image[index++]);
		
		  for (i=0; i<8; i++)
		  {
			if (data & (0x80>>i))
			{
				_Display_ImagePixel(w+i+x1, h+y1, 1);
			} else {
			    _Display_ImagePixel(w+i+x1, h+y1, 0);
			}
		  }
			 
	  }
	
	}	
	
}


void _Display_ImagePixel(int16_t x, int16_t y, uint8_t mode)
{
	
	if (dmLandscape == DisplayMode)
	{
		// Landscape
		if ( (x<0) || (y<0) || (x>=DISPLAY_WIDTH) || (y>=DISPLAY_HEIGHT) ) return;
		
		if (1 == mode)			display_memory[x][y>>3] |= 0x80>>(y & 0x07);
		else					display_memory[x][y>>3] &= ~(0x80>>(y & 0x07));
		
		
	} else {
		// Portrait
		if ( (x<0) || (y<0) || (y>=DISPLAY_WIDTH) || (x>=DISPLAY_HEIGHT) ) return;
		
		if (1 == mode)			display_memory[y][7-(x>>3)] |= 0x01<<(x & 0x07);
		else					display_memory[y][7-(x>>3)] &= ~(0x01<<(x & 0x07));
				
	}	
	
}



void Display_Update(void)
{	

   for (uint8_t page=0; page<8; page++) {
	   _Display_SendCommand( page | 0xb0 );
	   _Display_SendCommand(0x10);
	   _Display_SendCommand(0x00);
	   for (uint8_t col=0; col<128; col++) _Display_SendData(display_memory[col][7-page]);	     
   }    
		
}



void Display_SetContrast(uint8_t i)
{

	// i = 0...40, 20 center value

	if ( i>40 ) i = 40;
	i += 12;
	_Display_SendCommand(0x81);
	_delay_ms(5);
	_Display_SendCommand(i);
}


const uint8_t modulecfg[] PROGMEM =
{0xAE, 0x40, 0xA0, 0xC0, 0xA6, 0xA2, 0x2F, 0xF8, 0x00, 0x24, 0x81, 0x1F, 0xAF};
//AE40A0C0A6A22FF80024811FAF

void _Display_InitDisplayModule(void)
{

  DISPLAY_RESET_LOW
  _delay_ms(10);
  DISPLAY_RESET_HIGH
  _delay_ms(10);

  DISPLAY_CS_LOW

  _delay_ms(5);

  for (uint8_t i=0; i<14; i++)
  {
	  _Display_SendCommand(pgm_read_byte(&modulecfg[i]));
	  _delay_ms(10);	  
  }

}

void _Display_SendCommand(uint8_t cmd)
{

	DISPLAY_CS_LOW
	DISPLAY_A0_LOW

	_Display_SPI_Write(cmd);	
	
	DISPLAY_A0_HIGH
	DISPLAY_CS_HIGH
}

void _Display_SendData(uint8_t data)
{

	DISPLAY_CS_LOW
	DISPLAY_A0_HIGH
	
	_Display_SPI_Write(data);
	
	DISPLAY_CS_HIGH

}



void _Display_Char(int16_t x, int16_t y, char c, uint8_t width, uint8_t height)
{
	uint8_t xi, data, index;
	uint16_t data16;	
	
	switch (FontIndex)
	{
	  case FONT_1:	
	
		for (xi=0; xi<width; xi++ )
		{			
			data = pgm_read_byte(&font1[(uint8_t)c][xi]);
				
			for (uint8_t i=0; i<height; i++)
			{
				if (data & (0x80>>i))
					_Display_DrawPixelChar(x+xi, y+i, 0);	
				else
					_Display_DrawPixelChar(x+xi, y+i, 1);		
			}            
		}
		break;
		
	  #if EMBED_FONT2 == true
	  
	  case FONT_2:
	    index = 0;
		for (xi=0; xi<width; xi++ )
		{			
			data16 = (uint16_t)pgm_read_byte(&font2[(uint8_t)c][index++]);
			data16 |= (uint16_t)pgm_read_byte(&font2[(uint8_t)c][index++])<<8;
			
			for (uint8_t i=0; i<height; i++)
			{
			    if (data16 & (0x8000>>i))	
				  _Display_DrawPixelChar(x+xi, y+i, 0);
				else
				  _Display_DrawPixelChar(x+xi, y+i, 1);
							
			}            
		}
		break;	
				  
	  #endif
	  		
	  default:
		return;
		
    }	
}


void Display_Text(int16_t x, int16_t y, char *string)
{
	
	uint8_t width, height;
	
	switch (FontIndex)
	{
		
		case FONT_1:	width = 7;
						height = 8;
						break;
		case FONT_2:	width = 10;
						height = 16;
						break;
		default:		return;
	}
	
	while (string[0] != 0)
	{
	
		_Display_Char(x, y, string[0], width, height);
		x += width;
	    string++;	
		
	}
	
}

void Display_Text_F(int16_t x, int16_t y, const char *string)
{
	
	uint8_t width, height, c;
	
	switch (FontIndex)
	{
		
		case FONT_1:	width = 7;
		height = 8;
		break;
		case FONT_2:	width = 10;
		height = 16;
		break;
		default:		return;
	}
		

	while ( (c = pgm_read_byte(&(*string++))) ) 
	{
		_Display_Char(x, y, c, width, height);
		x += width;		
	}	
	

	
}


void Display_Decimal8(int16_t x, int16_t y, uint8_t d)
{
	char s[4];
	
	s[0] = '0' + (d / 100) % 10;
	s[1] = '0' + (d / 10) % 10;
	s[2] = '0' + d % 10;
	s[3] = 0;
	
	Display_Text(x, y, s);
	
}

void Display_Decimal16(int16_t x, int16_t y, uint16_t d)
{
	
	char s[6];
		
	s[0] = '0' + (d / 10000) % 10;		
	s[1] = '0' + (d / 1000) % 10;
	s[2] = '0' + (d / 100) % 10;
	s[3] = '0' + (d / 10) % 10;	
	s[4] = '0' + d % 10;
	s[5] = 0;
	
	Display_Text(x, y, s);
	
}